package com.isu.gaswellwatch.vo.command.scss;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import com.isu.gaswellwatch.vo.command.Timing;
import com.isu.gaswellwatch.vo.command.knpcv1.mode.PressureMode;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.List;
import java.util.Objects;

/**
 * 逻辑定值-简单模式定值数据寄存器地址
 *
 * @author <a href="mailto:scwsl@foxmail.com>王仕龙</a>
 * 2024/11/28 0:31
 */
@Getter
@Setter
@SuperBuilder
public class SimpleLogicMode extends Command implements Timing {
    @Serial
    private static final long serialVersionUID = 8899565585137994751L;
    /**
     * 开井时间，以秒为单位的时间
     */
    @NotBlank(message = "开井时间不能为空，格式：HH:mm:ss")
    private String wellOpenTimeTimestamp;
    /**
     * 开井套压
     */
    @NotNull(message = "开井套压不能为空")
    private BigDecimal wellOpenPressureValue = BigDecimal.ZERO;
    /**
     * 开井油压
     */
    @NotNull(message = "开井油压不能为空")
    private BigDecimal wellOpenOilPressureValue = BigDecimal.ZERO;
    /**
     * 开井载荷因子预设值
     */
    @NotNull(message = "开井载荷因子预设值不能为空")
    private BigDecimal wellOpenLoadFactorPresets = BigDecimal.ZERO;
    /**
     * 关井时间
     */
    @NotBlank(message = "关井时间不能为空，格式：HH:mm:ss")
    private String wellCloseTimeTimestamp;
    /**
     * 关井套压
     */
    @NotNull(message = "关井套压不能为空")
    private BigDecimal wellClosePressureValue = BigDecimal.ZERO;
    /**
     * 关井油压
     */
    @NotNull(message = "关井油压不能为空")
    private BigDecimal wellCloseOilPressureValue = BigDecimal.ZERO;
    /**
     * 关井流量
     */
    private int wellCloseFlowValue;
    /**
     * 最小开井时间
     */
    @NotBlank(message = "最小开井时间不能为空，格式：HH:mm:ss")
    private String minWellOpenTimeDuration;
    /**
     * 最大开井时间
     */
    @NotBlank(message = "最大开井时间不能为空，格式：HH:mm:ss")
    private String maxWellOpenTimeDuration;
    /**
     * 最小关井时间
     */
    @NotBlank(message = "最小关井时间不能为空，格式：HH:mm:ss")
    private String minWellCloseTimeDuration;
    /**
     * 最大关井时间
     */
    @NotBlank(message = "最大关井时间不能为空，格式：HH:mm:ss")
    private String maxWellCloseTimeDuration;
    /**
     * 压力稳定时间
     */
    @NotBlank(message = "压力稳定时间不能为空，格式：HH:mm:ss")
    private String pressureStabilizationDuration;
    /**
     * 流量稳定时间
     */
    @NotBlank(message = "流量稳定时间不能为空，格式：HH:mm:ss")
    private String flowStabilizationDuration;
    /**
     * 载荷因子稳定时间
     */
    @NotBlank(message = "载荷因子稳定时间不能为空，格式：HH:mm:ss")
    private String loadFactorStabilizationDuration;
    /**
     * 柱塞延迟时间
     */
    @NotBlank(message = "柱塞延迟时间不能为空，格式：HH:mm:ss")
    private String plungerDelayDuration;

    public SimpleLogicMode() {
        this.setCode("SCSS.SIMPLE_LOGIC_MODE");
    }

    @Override
    public boolean validate() {
        return super.validate()
                && Objects.nonNull(this.wellOpenPressureValue)
                && StringUtils.isNotBlank(this.plungerDelayDuration)
                && StringUtils.isNotBlank(this.wellOpenTimeTimestamp)
                && StringUtils.isNotBlank(this.wellCloseTimeTimestamp)
                && StringUtils.isNotBlank(this.minWellOpenTimeDuration)
                && StringUtils.isNotBlank(this.maxWellOpenTimeDuration)
                && StringUtils.isNotBlank(this.minWellCloseTimeDuration)
                && StringUtils.isNotBlank(this.maxWellCloseTimeDuration)
                && StringUtils.isNotBlank(this.flowStabilizationDuration)
                && StringUtils.isNotBlank(this.pressureStabilizationDuration)
                && StringUtils.isNotBlank(this.loadFactorStabilizationDuration)
                && this.wellOpenPressureValue.compareTo(BigDecimal.ZERO) >= 0
                && this.wellOpenPressureValue.multiply(PressureMode.MULTIPLE).compareTo(PressureMode.MAX_VALUE) <= 0
                && Objects.nonNull(this.wellOpenOilPressureValue)
                && this.wellOpenOilPressureValue.compareTo(BigDecimal.ZERO) >= 0
                && this.wellOpenOilPressureValue.multiply(PressureMode.MULTIPLE).compareTo(PressureMode.MAX_VALUE) <= 0
                && Objects.nonNull(this.wellOpenLoadFactorPresets)
                && this.wellOpenLoadFactorPresets.compareTo(BigDecimal.ZERO) >= 0
                && this.wellOpenLoadFactorPresets.multiply(PressureMode.MULTIPLE).compareTo(PressureMode.MAX_VALUE) <= 0;
    }

    @Override
    protected Collection<ModbusCommandDto> builderModbusCommand() {
        StringBuilder command = new StringBuilder(130);
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     0096    0020     40
        command.append("01100096002040");
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.wellOpenTimeTimestamp)), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellOpenPressureValue.multiply(PressureMode.MULTIPLE).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellOpenOilPressureValue.multiply(PressureMode.MULTIPLE).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellOpenLoadFactorPresets.multiply(PressureMode.MULTIPLE).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.wellCloseTimeTimestamp)), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellClosePressureValue.multiply(ONE_HUNDRED).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellCloseOilPressureValue.multiply(ONE_HUNDRED).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellCloseFlowValue), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.minWellOpenTimeDuration)), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.maxWellOpenTimeDuration)), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.minWellCloseTimeDuration)), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.maxWellCloseTimeDuration)), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.pressureStabilizationDuration)), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.flowStabilizationDuration)), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.loadFactorStabilizationDuration)), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.plungerDelayDuration)), 8, "0"));
        return List.of(ModbusCommandDto.builder().command(command.toString()).length(16).build());
    }

    public void setWellOpenTimeTimestamp(String wellOpenTimeTimestamp) {
        this.wellOpenTimeTimestamp = this.timingValidate(wellOpenTimeTimestamp, "wellOpenTimeTimestamp");
    }

    public void setWellCloseTimeTimestamp(String wellCloseTimeTimestamp) {
        this.wellCloseTimeTimestamp = this.timingValidate(wellCloseTimeTimestamp, "wellCloseTimeTimestamp");
    }

    public void setMinWellOpenTimeDuration(String minWellOpenTimeDuration) {
        this.minWellOpenTimeDuration = this.timingValidate(minWellOpenTimeDuration, "minWellOpenTimeDuration");
    }

    public void setMaxWellOpenTimeDuration(String maxWellOpenTimeDuration) {
        this.maxWellOpenTimeDuration = this.timingValidate(maxWellOpenTimeDuration, "maxWellOpenTimeDuration");
    }

    public void setMinWellCloseTimeDuration(String minWellCloseTimeDuration) {
        this.minWellCloseTimeDuration = this.timingValidate(minWellCloseTimeDuration, "minWellCloseTimeDuration");
    }

    public void setMaxWellCloseTimeDuration(String maxWellCloseTimeDuration) {
        this.maxWellCloseTimeDuration = this.timingValidate(maxWellCloseTimeDuration, "maxWellCloseTimeDuration");
    }

    public void setPressureStabilizationDuration(String pressureStabilizationDuration) {
        this.pressureStabilizationDuration = this.timingValidate(pressureStabilizationDuration, "pressureStabilizationDuration");
    }

    public void setFlowStabilizationDuration(String flowStabilizationDuration) {
        this.flowStabilizationDuration = this.timingValidate(flowStabilizationDuration, "flowStabilizationDuration");
    }

    public void setLoadFactorStabilizationDuration(String loadFactorStabilizationDuration) {
        this.loadFactorStabilizationDuration = this.timingValidate(loadFactorStabilizationDuration, "loadFactorStabilizationDuration");
    }

    public void setPlungerDelayDuration(String plungerDelayDuration) {
        this.plungerDelayDuration = this.timingValidate(plungerDelayDuration, "plungerDelayDuration");
    }
    
}
